<?php

namespace Supablox;

const ASSETS_DIR = DIR . '/build';
const ASSETS_URL = URL . 'build/';

add_action( 'init', __NAMESPACE__ . '\register_assets', 0 );

function register_assets() {
	if ( is_development() ) {
		register_script( 'supablox/runtime', 'runtime.js' );
	}

	register_script( 'supablox/block-editor', 'block-editor.js' );
	register_script( 'supablox/editor', 'editor.js' );
	register_script( 'supablox/global-styles', 'global-styles.js' );
	register_script( 'supablox/wp-block-editor', 'wp-block-editor.js' );
	register_script( 'supablox/wp-components', 'wp-components.js' );
	register_script( 'supablox/wp-plugins', 'wp-plugins.js' );
	register_script( 'supablox/wp-private-apis', 'wp-private-apis.js' );

	register_style( 'supablox/admin', 'admin.css' );
	register_style( 'supablox/block-editor', 'block-editor.css' );
	register_style( 'supablox/wp-block-editor', 'wp-block-editor.css' );
	register_style( 'supablox/wp-components', 'wp-components.css' );
	register_style(
		'supablox/editor',
		'editor.css',
		[
			'dashicons',
			'wp-components',
			'wp-block-editor',
			'wp-block-editor-content',
			'wp-format-library',
			'supablox/admin',
			'supablox/wp-block-editor',
			'supablox/wp-components',
		]
	);
}

/**
 * @param string $handle
 * @param string $filename
 * @param array  $dependencies
 */
function register_script( $handle, $filename, $dependencies = [] ) {
	$assets_info = get_asset_info( $filename );

	if ( $handle !== 'supablox/runtime' && is_development() ) {
		$dependencies[] = 'supablox/runtime';
	}

	wp_register_script(
		$handle,
		ASSETS_URL . $filename,
		array_merge( $assets_info['dependencies'], $dependencies ),
		$assets_info['version'],
		true
	);
}

function register_style( string $handle, string $filename, array $dependencies = [] ) {
	$version = VERSION;

	if ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) {
		if ( file_exists( ASSETS_DIR . "/$filename" ) ) {
			$version = filemtime( ASSETS_DIR . "/$filename" );
		}
	}

	wp_register_style(
		$handle,
		ASSETS_URL . $filename,
		$dependencies,
		$version
	);
}

/**
 * @return array{dependencies:string[],version:string>
 */
function get_asset_info( string $filename ) {
	$asset = str_replace( '.js', '', $filename );
	$path = ASSETS_DIR . "/$asset.asset.php";

	if ( file_exists( $path ) ) {
		return require $path;
	}

	return [
		'version' => VERSION,
		'dependencies' => [],
	];
}
