<?php

namespace Supablox;

add_filter( 'register_block_type_args', __NAMESPACE__ . '\register_block_type_args', 10, 2 );

function register_block_type_args( array $args, string $block_type ) {
	if ( isset( $args['supports']['supablox'] ) ) {
		return $args;
	}

	$unsupported_block_types = [
		'core/pattern' => is_wp_version( '6.6' ),
		// This block has CSS selectors that greatly hurt performance in the editor.
		'woocommerce/coming-soon' => is_editor() || is_canvas(),
	];

	if ( ! empty( $unsupported_block_types[ $block_type ] ) ) {
		$args['style_handles'] = [];
		$args['editor_style_handles'] = [];
		$args['editor_script_handles'] = [];
		return $args;
	}

	if ( isset( $args['api_version'] ) && $args['api_version'] >= 2 ) {
		$support = [];

		switch ( $block_type ) {
			case 'core/columns':
				$support['droppable'] = [
					'accept' => true,
					'wrapperBlock' => [
						'blockName' => 'core/column',
					],
				];
				break;
			case 'core/button':
				$support['summary'] = '{{ text }}';
				break;
			case 'core/paragraph':
				$support['summary'] = '{{ content }}';
				break;
			case 'core/spacer':
				$support['summary'] = '{{ height }}';
				break;
		}

		$args['supports']['supablox'] = $support;
	}

	return $args;
}
