<?php

namespace Supablox;

class REST_Preview_Controller {

	public function register_routes() {
		register_rest_route(
			'supablox/v1',
			'preview',
			[
				'methods' => 'POST',
				'permission_callback' => [ $this, 'can_edit_post' ],
				'callback' => [ $this, 'post_preview' ],
				'args' => [
					'records' => [
						'type' => 'object',
						'required' => true,
					],
					'post_id' => [
						'type' => [ 'string', 'number' ],
					],
				],
			]
		);
	}

	public function can_edit_post() {
		return current_user_can( 'edit_posts' );
	}

	public function post_preview( \WP_REST_Request $req ) {
		delete_transient( get_preview_key() );

		if ( ! empty( $req['records'] ) ) {
			set_transient( get_preview_key(), $req['records'], 60 * 60 );
		}

		$preview_link = null;

		if ( isset( $req['post_id'] ) ) {
			if ( is_string( $req['post_id'] ) && strpos( $req['post_id'], '//' ) !== false ) {
				$slug = explode( '//', $req['post_id'] )[1];

				switch ( $slug ) {
					case 'home':
						$preview_link = add_query_arg( 'preview', true, home_url( '/' ) );
						break;
				}
			} elseif ( is_numeric( $req['post_id'] ) ) {
				$preview_link = get_preview_post_link(
					$req['post_id'],
					[
						'preview_id' => $req['post_id'],
						'preview_nonce' => wp_create_nonce( 'post_preview_' . $req['post_id'] ),
					]
				);
			}
		}

		return wp_send_json_success(
			[
				'preview_link' => $preview_link,
			]
		);
	}
}
