<?php

namespace Supablox;

class REST_Settings_Controller {

	public function register_routes() {
		register_rest_route(
			'supablox/v1',
			'settings',
			[
				'methods' => 'GET',
				'permission_callback' => [ $this, 'can_edit_post' ],
				'callback' => [ $this, 'get_settings' ],
				'args' => [
					'post_type' => [
						'type' => 'string',
						'required' => true,
					],
					'post_id' => [
						'type' => [ 'number', 'string' ],
						'required' => true,
					],
				],
			]
		);
	}

	public function can_edit_post( \WP_REST_Request $req ) {
		$post_type = $req->get_param( 'post_type' );
		$post_id = $req->get_param( 'post_id' );

		if ( $post_type === 'wp_template' ) {
			return current_user_can( 'edit_theme_options' );
		} elseif ( is_numeric( $post_id ) ) {
			return current_user_can( 'edit_post', $post_id );
		}

		return current_user_can( 'edit_posts' );
	}

	public function get_settings( \WP_REST_Request $req ) {
		return rest_ensure_response(
			get_editor_settings_for_post(
				$req->get_param( 'post_type' ),
				$req->get_param( 'post_id' )
			)
		);
	}
}
