<?php

namespace Supablox;

function require_package( string $path, callable $callback = null ) {
	static $paths = [];

	if ( file_exists( $path ) ) {
		require_once $path;

		if ( $callback ) {
			$callback();
		}

		return;
	}

	$paths[] = $path;

	add_action(
		'admin_head',
		function () use ( &$paths ) {
			if ( ! empty( $paths ) ) {
				$message = '<strong>Required Supablox packages not found:</strong>';
				$message .= '<ul>';
				foreach ( $paths as $error ) {
					$message .= '<li>- ' . esc_html( $error ) . '</li>';
				}
				$message .= '</ul>';
				wp_admin_notice( $message, [ 'type' => 'error' ] );
				$paths = [];
			}
		}
	);
}

function is_wp_version( string $version ) {
	return version_compare( get_bloginfo( 'version' ), $version, '>=' );
}

function is_development() {
	return defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG && file_exists( __DIR__ . '/../build/runtime.js' );
}

function is_editor() {
	global $pagenow;

	return (
		$pagenow === 'index.php' &&
		isset( $_GET['page'] ) && // phpcs:ignore WordPress.Security
		$_GET['page'] === 'supablox' // phpcs:ignore WordPress.Security
	);
}

function is_canvas() {
	return isset( $_GET['supablox'] ) && boolval( $_GET['supablox'] ); // phpcs:ignore WordPress.Security
}

function is_preview() {
	return isset( $_GET['preview'] ) && boolval( $_GET['preview'] ); // phpcs:ignore WordPress.Security
}

function get_editor_url( array $args = [] ) {
	return add_query_arg(
		array_merge(
			[
				'page' => 'supablox',
			],
			$args
		),
		admin_url( 'index.php' )
	);
}

/**
 * @return array<int,WP_Post_Type>
 */
function get_supported_post_types(): array {
	return array_filter(
		array_map(
			fn ( $post_type ) => get_post_type_object( $post_type ),
			get_post_types_by_support( 'supablox' )
		)
	);
}
