<?php

namespace Supablox\Packages\Admin;

class Cron {

	private string $event_name;

	public function __construct() {
		$this->event_name = PACKAGE_NAME . '_validate_license';
	}

	public function init() {
		add_action( 'init', [ $this, 'schedule_event' ] );
		add_action( $this->event_name, [ $this, 'validate' ] );
	}

	public function schedule_event() {
		if ( ! wp_next_scheduled( $this->event_name ) ) {
			wp_schedule_event( time(), 'daily', $this->event_name );
		}
	}

	public function clear_scheduled_event() {
		if ( wp_next_scheduled( $this->event_name ) ) {
			wp_clear_scheduled_hook( $this->event_name );
		}
	}

	public function validate() {
		$license = License::get_instance();
		$data = $license->validate();

		if ( is_wp_error( $data ) ) {
			$code = $data->get_error_code();
			$data = $data->get_error_data();
			$status = $data['status'] ?? null;

			if ( $code === 'invalid' || $status === 404 ) {
				$license->set_error( $data['error'] ?? 'Invalid license.' );
			}
			return;
		}
	}
}
