<?php

namespace Supablox\Packages\Admin;

class Supablox_Api {

	private static $instance = null;

	public function init() {
		add_filter( 'http_headers_useragent', [ $this, 'http_headers_useragent' ], 10, 2 );
	}

	public function http_headers_useragent( string $useragent, string $url = '' ) {
		if (
			str_contains( $url, SUPABLOX_API_URL ) &&
			preg_match( '/plugin\/' . PACKAGE_NAME . '(\/|\?|$)/', $url )
		) {
			$name = str_replace( '-', '', ucwords( PACKAGE_NAME, '-' ) );
			return $name . '/' . PACKAGE_VERSION . '; ' . $useragent;
		}
		return $useragent;
	}

	public function get_license( array $args = [] ) {
		return $this->request( 'GET', '/v1/plugin/' . PACKAGE_NAME . '/license', $args );
	}

	public function activate_license( array $args = [] ) {
		return $this->request( 'POST', '/v1/plugin/' . PACKAGE_NAME . '/license', $args );
	}

	public function deactivate_license( array $args = [] ) {
		return $this->request( 'DELETE', '/v1/plugin/' . PACKAGE_NAME . '/license', $args );
	}

	public function get_latest_version( array $args = [] ) {
		return $this->request( 'GET', '/v1/plugin/' . PACKAGE_NAME . '/latest', $this->get_request_args( $args ) );
	}

	public function get_version( string $version, array $args = [] ) {
		return $this->request( 'GET', '/v1/plugin/' . PACKAGE_NAME . '/' . $version, $args );
	}

	public function get_plugin_info() {
		return $this->request( 'GET', '/v1/plugin/' . PACKAGE_NAME, $this->get_request_args() );
	}

	private function get_request_args( array $args = [] ): array {
		$options = get_option( PACKAGE_NAME );
		$release_channel = $options['release_channel'] ?? 'stable';
		$prerelease = $options['prerelease'] ?? false;

		if ( $release_channel !== 'stable' ) {
			$args['channel'] = $release_channel;
		}

		if ( $prerelease === true ) {
			$args['canary'] = 'true';
		}

		return $args;
	}

	private function request( string $method, string $path, array $body = null ): array|\WP_Error {
		if ( $method === 'GET' && ! empty( $body ) ) {
			$path .= '?' . http_build_query( $body );
			$body = null;
		}

		$response = wp_remote_request(
			SUPABLOX_API_URL . $path,
			[
				'body' => $body,
				'method' => $method,
				'timeout' => 10,
				'sslverify' => false,
				'headers' => [
					'Referer' => home_url(),
				],
			]
		);

		if ( is_wp_error( $response ) ) {
			return $response;
		}

		$response_code = wp_remote_retrieve_response_code( $response );
		$body = wp_remote_retrieve_body( $response );
		$data = json_decode( $body, true );

		if ( isset( $data['code'] ) ) {
			return new \WP_Error( $data['code'], $data['message'], [ 'status' => $response_code ] );
		} elseif ( $response_code >= 400 ) {
			return new \WP_Error( 'error', 'An error occurred.', [ 'status' => $response_code ] );
		}

		return $data;
	}

	public static function get_instance(): static {
		if ( self::$instance === null ) {
			self::$instance = new self();
		}
		return self::$instance;
	}
}
