<?php

namespace Supablox\Packages\Controls;

class Rest_Controller {

	public function register_routes() {
		register_rest_route(
			'v1',
			'supablox/controls/display-value/post/(?P<ids>.*)',
			[
				'methods' => \WP_REST_Server::READABLE,
				'callback' => [ $this, 'get_post_display_value' ],
				'permission_callback' => '__return_true',
			]
		);

		register_rest_route(
			'v1',
			'supablox/controls/display-value/term/(?P<ids>.*)',
			[
				'methods' => \WP_REST_Server::READABLE,
				'callback' => [ $this, 'get_term_display_value' ],
				'permission_callback' => '__return_true',
			]
		);
	}

	public function get_post_display_value( \WP_REST_Request $req ) {
		$ids_param = $req->get_param( 'ids' );

		$ids = array_map( 'intval', explode( ',', $ids_param ) );

		$posts = get_posts(
			[
				'post__in' => $ids,
				'post_type' => get_post_types(),
				'orderby' => 'post__in',
			]
		);

		return array_map(
			function ( $post ) {
				return [
					'value' => $post->ID,
					'label' => $post->post_title,
					'meta' => [
						'subtype' => $post->post_type,
						'url' => get_permalink( $post->ID ),
					],
				];
			},
			$posts
		);
	}

	public function get_term_display_value( \WP_REST_Request $req ) {
		$ids_param = $req->get_param( 'ids' );

		$ids = array_map( 'intval', explode( ',', $ids_param ) );

		$terms = get_terms(
			[
				'include' => $ids,
				'orderby' => 'include',
			]
		);

		return array_map(
			function ( $term ) {
				return [
					'value' => $term->term_id,
					'label' => $term->name,
					'meta'  => [
						'subtype' => $term->taxonomy,
						'url' => get_term_link( $term->term_id ),
					],
				];
			},
			$terms
		);
	}
}
