<?php

namespace Supablox\Packages\Style_Engine;

class Attribute {

	private string $block_attribute_name;

	public string $name = '';

	public string $type = 'string';

	public bool $responsive = true;

	public bool $output = true;

	public string $class_name = '';

	public string|null $property = null;

	public $default = null;

	public $resolve_callback = null;

	public $should_output_callback = null;

	public function __construct(
		private Style_Engine $style_engine,
		string $name,
		array|bool $config = []
	) {
		$this->block_attribute_name = $style_engine->options['block_attribute_name'];
		$this->name = $name;

		if ( $config === false ) {
			$this->responsive = false;
		} else {
			foreach ( get_object_vars( $this ) as $key => $value ) {
				if ( isset( $config[ $key ] ) ) {
					$this->$key = $config[ $key ];
				}
			}
		}

		if ( empty( $this->property ) ) {
			$this->property = css_property_name( $this->name );
		}

		if ( ! is_null( $this->default ) && ! is_array( $this->default ) ) {
			$this->default = [
				'default' => [
					'lg' => $this->default,
				],
			];
		}
	}

	public function is_customized( array $attrs ) {
		foreach ( Registry::$states as $state ) {
			foreach ( Devices::get_ids() as $breakpoint ) {
				if ( isset( $attrs[ $this->block_attribute_name ]['style'][ $state ][ $this->name ][ $breakpoint ] ) ) {
					return true;
				}
			}
		}
		return false;
	}

	public function set_value( array &$attrs, string $breakpoint, string $state, mixed $value ) {
		$attrs[ $this->block_attribute_name ]['style'][ $state ][ $this->name ][ $breakpoint ] = $value;
	}

	public function get_value( array $attrs, string $breakpoint, string $state, string $context = 'raw' ) {
		$breakpoint = $this->responsive ? $breakpoint : Devices::get_default_id();
		$value = $attrs[ $this->block_attribute_name ]['style'][ $state ][ $this->name ][ $breakpoint ] ?? null;

		if ( $value && $context !== 'raw' ) {
			$value = $this->resolve_value( $value );
		}

		return $value;
	}

	public function get_default_value( string $breakpoint, string $state, string $context = 'raw' ) {
		$value = $this->default[ $state ][ $breakpoint ] ?? null;

		if (
			$context === 'output' &&
			$this->is_empty( $value ) &&
			$this->is_default_state( $breakpoint, $state )
		) {
			return 'initial';
		}

		if ( $context !== 'raw' ) {
			$value = $this->resolve_value( $value );
		}

		return $value;
	}

	private function resolve_value( $value ) {
		if ( is_callable( $this->resolve_callback ) ) {
			$value = call_user_func( $this->resolve_callback, $value );
		}

		if ( ! is_string( $value ) ) {
			return $value;
		}

		if ( preg_match( '/^var:preset\|(.+)/', $value, $match ) ) {
			return 'var(--wp--preset--' . join( '--', $this->to_kebab_case( explode( '|', $match[1] ) ) ) . ')';
		} elseif ( preg_match( '/^attachment:(\d+)$/', $value, $match ) ) {
			$url = wp_get_attachment_url( $match[1] );
			return $url ? 'url(' . esc_url( $url ) . ')' : 'none';
		}

		return $value;
	}

	private function to_kebab_case( array $strings ) {
		return array_map( __NAMESPACE__ . '\to_kebab_case', $strings );
	}

	private function is_empty( $value ) {
		return (
			is_null( $value ) ||
			( is_string( $value ) && trim( $value ) === '' ) ||
			( is_array( $value ) && empty( $value ) )
		);
	}

	private function is_default_state( string $breakpoint, string $state ) {
		return $breakpoint === 'lg' && $state === 'default';
	}

	public function to_json() {
		return [
			'type' => $this->type,
			'responsive' => $this->responsive,
			'output' => $this->output,
			'className' => $this->class_name,
			'property' => $this->property,
			'default' => $this->default,
		];
	}
}
