<?php

namespace Supablox\Packages\Style_Engine;

define( 'Supablox\Packages\Style_Engine\CSS_SPACE', defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? ' ' : '' );
define( 'Supablox\Packages\Style_Engine\CSS_EOL', defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? "\n" : '' );

class CSS_Utils {

	private string $class_name_prefix;
	private string $property_prefix;

	public function __construct( array $options ) {
		$this->class_name_prefix = $options['class_name_prefix'];
		$this->property_prefix = $options['property_prefix'];
	}

	public function at_rule( $id, array $options, array $rulesets ) {
		$content = $this->rulesets( $rulesets, 2 );

		if ( empty( $content ) ) {
			return '';
		}

		$css = '@' . sanitize_key( $id ) . CSS_SPACE;

		foreach ( $options as $option => $value ) {
			if ( is_null( $value ) || $value === '' ) {
				continue;
			}

			if ( $option !== array_key_first( $options ) ) {
				$css .= CSS_SPACE . 'and' . CSS_SPACE;
			}

			$css .= '(' . $option . ':' . CSS_SPACE . $value . ')';
		}

		return $css . CSS_SPACE . '{' . CSS_EOL . $content . CSS_EOL . '}';
	}

	public function rulesets( array $rulesets, $indents = 0 ) {
			$css = '';

			foreach ( $rulesets as $selector => $declarations ) {
				if ( empty( $declarations ) ) {
					continue;
				}

				$value = $this->ruleset( $selector, $declarations, $indents );

				if ( ! empty( $value ) && $selector !== array_key_last( $rulesets ) ) {
					$value .= CSS_EOL;
				}

				$css .= $value;
			}

			return rtrim( $css, CSS_EOL );
	}

	public function ruleset( $selector, array $declarations, $indents = 0 ) {
			$content = $this->declarations( $declarations, $indents + 2 );

			if ( empty( $content ) ) {
				return '';
			}

			$indent = str_repeat( CSS_SPACE, $indents );
			$css = $indent . $this->esc_selector( $selector ) . CSS_SPACE . '{' . CSS_EOL;
			$css .= $content . CSS_EOL;
			$css .= $indent . '}';

			return $css;
	}

	public function declarations( array $declarations, $indents = 0 ) {
			$css = '';

			foreach ( $declarations as $property => $value ) {
				if ( ! is_null( $value ) && $value !== '' ) {
					$css .= $this->declaration( $property, $value, $indents ) . CSS_EOL;
				}
			}

			return rtrim( $css, CSS_EOL );
	}

	public function declaration( $property, $value, $indents = 0 ) {
			$indent = str_repeat( CSS_SPACE, $indents );
			$declaration = $indent . $this->esc_property( $property ) . ':';
			$declaration .= CSS_SPACE . $this->esc_value( $value ) . ';';
			return $declaration;
	}

	public function esc_selector( $selector ) {
			return esc_js( $selector );
	}

	public function esc_property( $property ) {
			return sanitize_key( $property );
	}

	public function class_name( $name ) {
			return $this->class_name_prefix . $name;
	}

	public function property_name( $name ) {
			return $this->property_prefix . to_kebab_case( $name );
	}

	public function to_variable( $value ) {
			if ( is_string( $value ) && str_starts_with( $value, 'var:' ) ) {
				return 'var(--wp--' . str_replace( '|', '--', substr( $value, 4 ) ) . ')';
			}
			return $value;
	}

	public function esc_value( $value ) {
			return is_numeric( $value ) ? $value : sanitize_text_field( $value );
	}
}
