<?php

namespace Supablox\Packages\Style_Engine;

class Group_Hover {

	public string $block_support_name;
	public string $attribute_name = 'groupHover';
	public string $context_name;

	public function __construct( private Style_Engine $style_engine ) {
		$this->context_name = $style_engine->options['id'] . '/groupHover';
		$this->block_support_name = $style_engine->options['block_support_prefix'] . 'GroupHover';

		\WP_Block_Supports::get_instance()->register(
				$this->block_support_name,
				[
					'register_attribute' => [ $this, 'register_attribute' ],
				]
		);

		add_filter( 'register_block_type_args', [ $this, 'register_block_type_args' ], 10 );
		add_filter( 'render_block_data', [ $this, 'render_block_data' ] );
		add_filter( 'render_block_context', [ $this, 'render_block_context' ], 10, 3 );
	}


	public function register_attribute( \WP_Block_Type $block_type ) {
		if ( ! block_has_support( $block_type, $this->block_support_name, false ) ) {
			return;
		}

		$block_type->attributes[ $this->attribute_name ] = [
			'type' => [
				'boolean', // A boolean when enabled in the editor.
				'string', // A class name when provided as context.
			],
		];

		$block_type->provides_context[ $this->context_name ] = $this->attribute_name;
	}

	public function register_block_type_args( array $args ) {
		if ( ! isset( $args['uses_context'] ) ) {
			$args['uses_context'] = [];
		}
		if ( ! in_array( $this->context_name, $args['uses_context'], true ) ) {
			$args['uses_context'][] = $this->context_name;
		}
		return $args;
	}

	public function render_block_data( array $block ) {
		if ( isset( $block['attrs'][ $this->attribute_name ] ) && $block['attrs'][ $this->attribute_name ] ) {
			$block['attrs'][ $this->attribute_name ] = $this->get_class_name();
		}
		return $block;
	}

	public function render_block_context( array $context, array $block, \WP_Block $parent_block = null ) {
		if ( ! $parent_block ) {
			return $context;
		}

		if (
			isset( $parent_block->parsed_block['attrs'][ $this->attribute_name ] ) &&
			$parent_block->parsed_block['attrs'][ $this->attribute_name ]
		) {
			$context[ $this->context_name ] = $parent_block->parsed_block['attrs'][ $this->attribute_name ];
		} elseif ( isset( $parent_block->context[ $this->context_name ] ) ) {
			$context[ $this->context_name ] = $parent_block->context[ $this->context_name ];
		}

		return $context;
	}

	private function get_class_name() {
		return $this->style_engine->css_utils->class_name( 'group-' . get_increment_for( 'group' ) );
	}
}
