<?php

namespace Supablox\Packages\Style_Engine;

class Registry {

	public static $states = [ 'default', 'hover', 'groupHover' ];

	private string $block_attribute_name;

	/** @var array<string,array<string,Attribute>> */
	private $registered = [];

	/** @var array<string,array<string,Attribute>> */
	public array $enqueued_styles = [];

	/** @var array<string,array{0:\WP_Block,1:array<string,Attribute>}> */
	public array $enqueued_customized_styles = [];

	public function __construct( private Style_Engine $style_engine ) {
		$this->block_attribute_name = $style_engine->options['block_attribute_name'];

		add_action( 'init', [ $this, 'register_attributes' ], 23 ); // After WP_Block_Supports::init().
	}

	public function enqueue_style( string $block_name, array $style_attributes ) {
		$this->enqueued_styles[ $block_name ] = $style_attributes;
	}

	public function enqueue_customized_style( string $class_name, \WP_Block $block, array $style_attributes ) {
		$this->enqueued_customized_styles[ $class_name ] = [ $block, $style_attributes ];
	}

	public function register( \WP_Block_Type $block_type, array $style_attributes ) {
		foreach ( $style_attributes as $attr_name => $attr_config ) {
			$attr = new Attribute( $this->style_engine, $attr_name, $attr_config );
			$this->registered[ $block_type->name ][ $attr_name ] = $attr;
		}
	}

	public function is_registered( $block_name ) {
		return isset( $this->registered[ $block_name ] );
	}

	public function get_all_registered() {
		return $this->registered;
	}

	public function get_registered( $block_name ) {
		return $this->is_registered( $block_name )
			? $this->registered[ $block_name ]
			: null;
	}

	public function resolve_value( array $value, string $breakpoint, $fallback_value = null ) {
		$ids = Devices::get_ids();
		$index = array_search( $breakpoint, $ids, true );
		$default_index = Devices::get_default_index();
		$values = [
			$value['sm'] ?? $value['md'] ?? $fallback_value,
			$value['md'] ?? $value['lg'] ?? $fallback_value,
			$value['lg'] ?? $fallback_value,
			$value['xl'] ?? $value['lg'] ?? $fallback_value,
		];

		if ( $index > $default_index ) {
			for ( $i = $index; $i >= 0; $i-- ) {
				if ( ! is_null( $values[ $i ] ) ) {
					return $values[ $i ];
				}
			}
			return null;
		}

		$length = count( $values );

		for ( $i = $index; $i < $length; $i++ ) {
			if ( ! is_null( $values[ $i ] ) ) {
				return $values[ $i ];
			}
		}

		return null;
	}

	public function register_attributes() {
		$block_registry = \WP_Block_Type_Registry::get_instance();

		foreach ( $block_registry->get_all_registered() as $block_type ) {
			if (
				! ( $block_type instanceof \WP_Block_Type ) ||
				! $this->is_registered( $block_type->name )
			) {
				continue;
			}

			if ( ! $block_type->attributes ) {
				$block_type->attributes = [];
			}

			if ( ! isset( $block_type->attributes[ $this->block_attribute_name ] ) ) {
				$block_type->attributes[ $this->block_attribute_name ] = [ 'type' => 'object' ];
			}
		}
	}

	public function reset() {
		$this->registered = [];
		$this->enqueued_styles = [];
		$this->enqueued_customized_styles = [];
	}

	public function to_json() {
		return array_map(
			function ( $attrs ) {
				return array_map(
					fn ( $attr ) => $attr->to_json(),
					$attrs
				);
			},
			$this->registered
		);
	}
}
