<?php

namespace Supablox\Packages\Style_Engine;

class Style_Engine {

	/** @var int */
	public $id;

	/** @var Registry */
	public $registry;

	/** @var Group_Hover */
	public $group_hover;

	/** @var Inline_Styles */
	public $inline_styles;

	/** @var CSS_Utils */
	public $css_utils;

	private function __construct( public array $options ) {
		$this->id = $options['id'];
		$this->registry = new Registry( $this );
		$this->group_hover = new Group_Hover( $this );
		$this->inline_styles = new Inline_Styles( $this );
		$this->css_utils = new CSS_Utils(
			[
				'class_name_prefix' => $options['css_class_name_prefix'],
				'property_prefix' => $options['css_property_prefix'],
			]
		);
	}

	public function register( \WP_Block_Type $block_type, array $style_attributes ) {
		return $this->registry->register( $block_type, $style_attributes );
	}

	public function get_data() {
		return [
			'id' => $this->options['id'],
			'breakpoints' => Devices::get_all(),
			'styleAttributes' => (object) $this->registry->to_json(),
			'blockCss' => $this->inline_styles->get_default_block_type_css(),
			'blockSupportPrefix' => $this->options['block_support_prefix'],
			'blockAttributeName' => $this->options['block_attribute_name'],
			'cssClassNamePrefix' => $this->options['css_class_name_prefix'],
			'cssPropertyPrefix' => $this->options['css_property_prefix'],
		];
	}

	private static $instance = null;

	public static function get_instance( array $options = null ): static {
		if ( ! self::$instance && is_array( $options ) ) {
			self::$instance = new self( $options );
		}
		return self::$instance;
	}
}
