<?php

namespace Supablox;

add_filter( 'get_edit_post_link', __NAMESPACE__ . '\post_takeover_redirect', 10, 3 );

function is_post_type_supported( string $post_type ) {
	if ( ! post_type_supports( $post_type, 'supablox' ) ) {
		return false;
	}

	$post_type_object = get_post_type_object( $post_type );
	if ( $post_type_object && ! $post_type_object->show_in_rest ) {
		return false;
	}

	return true;
}

function is_page_on_front( array $post ) {
	return (int) get_option( 'page_on_front' ) === (int) $post['id'];
}

function is_page_for_posts( array $post ) {
	return (int) get_option( 'page_for_posts' ) === (int) $post['id'];
}

function get_edit_post_link( int|string $post_id ) {
	$post = get_post( $post_id );

	if ( ! $post || ! post_type_supports( $post->post_type, 'supablox' ) ) {
		return null;
	}

	return get_editor_url(
		[
			'postType' => $post->post_type,
			'postId' => $post_id,
		]
	);
}

function get_post_lock_details( \WP_Post $post ) {
	require_once ABSPATH . 'wp-admin/includes/post.php';

	$user_id = wp_check_post_lock( $post->ID );
	$lock_details = null;

	if ( $user_id ) {
		if ( apply_filters( 'show_post_locked_dialog', true, $post, $user_id ) ) {
			$user = get_userdata( $user_id );
			$lock_details = [
				'isLocked' => true,
				'user' => [
					'avatar' => get_avatar_url( $user_id, [ 'size' => 128 ] ),
					'name' => $user->display_name,
				],
			];
		}
	} else {
		$active_post_lock = wp_set_post_lock( $post->ID );
		if ( $active_post_lock ) {
			$active_post_lock = esc_attr( implode( ':', $active_post_lock ) );
		}
		$lock_details = [
			'isLocked' => false,
			'activePostLock' => $active_post_lock,
		];
	}

	return $lock_details;
}

function post_takeover_redirect( string $link, int $post_id, string $context ) {
	if (
		isset( $_GET['get-post-lock'] ) && // phpcs:ignore WordPress.Security
		isset( $_GET['post'] ) && // phpcs:ignore WordPress.Security
		(int) $_GET['post'] === $post_id // phpcs:ignore WordPress.Security
	) {
		return get_editor_url(
			[
				'postType' => get_post_type( $post_id ),
				'postId' => $post_id,
			]
		);
	}
	return $link;
}
